#include "Interpolator2D.h"

Interpolator2D::Interpolator2D(const std::string& fileName) {

	std::ifstream inputValuesFile;
	inputValuesFile.open(fileName);
	if (!inputValuesFile.is_open()) {
		std::cerr << "Interpolator2D::Interpolator2D Error opening file: " << fileName << std::endl;
	}

	// The first two file entries contain the number of points in the x and y dimensions:
	inputValuesFile >> dimX;
	inputValuesFile >> dimY;

	xInput.reserve(dimX);
	yInput.reserve(dimY);
	zInput.reserve(dimX*dimY);

	for (unsigned i=0; i<dimX*dimY; i++) {
		inputValuesFile >> xInput[i/dimY];
		inputValuesFile >> yInput[i%dimY];
		inputValuesFile >> zInput[i];
	}

	// create (bilinear) interpolator
	accX = gsl_interp_accel_alloc();
	accY = gsl_interp_accel_alloc();
	bilinear = gsl_spline2d_alloc(gsl_interp2d_bilinear, dimY, dimX);
	gsl_spline2d_init(bilinear, &yInput[0], &xInput[0], &zInput[0], dimY, dimX);

	inputValuesFile.close();
}


Interpolator2D::~Interpolator2D() {
	gsl_spline2d_free(bilinear);
	gsl_interp_accel_free(accX);
	gsl_interp_accel_free(accY);
}


double Interpolator2D::getZ(const double x, const double y) {
	// check for out of bounds:
	if (x < xInput[0] || x > xInput[dimX-1] || y < yInput[0] || y > yInput[dimY-1]) {
		return 0.0;
	}
	return gsl_spline2d_eval(bilinear, y, x, accY, accX);
}
