/*
 Use a 2D interpolator from GSL to interpolate a pre-evaluated grid of
 Vfin points, and their errors. As a command line argument, pass the
 name of a file containing the desired sqrts,pt pairs.

 To compile:
 g++ Interpolator2D.cpp Vfin.cpp -o Vfin -lm -lgsl -lgslcblas
*/

#include <iostream>
#include <fstream>
#include <cmath>
#include <vector>
#include "Interpolator2D.h"


int main(int argc, char *argv[])
{
	// Physics constants
	const double mz = 91.1876;


	// Read evaluation points from command-line-parameter specified file, which
	// contains "sqrts pt" pairs.
	if (argc < 2) {
		std::cerr << "Error, specify evaluation points file as an argument." << std::endl;
		return 1;
	}
	std::ifstream evalPointsFile;
	evalPointsFile.open(argv[1]);
	if (!evalPointsFile.is_open()) {
		std::cerr << "Error, could not open file: " << argv[1] << std::endl;
		return 1;
	}

	std::vector<double> sqrtsEval;
	std::vector<double> ptEval;
	while (evalPointsFile.good()) {
		double sqrts, pt;
		evalPointsFile >> sqrts >> pt;
		sqrtsEval.push_back(sqrts);
		ptEval.push_back(pt);
	}


	Interpolator2D valueInterp("src/grid/Vfin_grid.dat");
	Interpolator2D errorInterp("src/grid/Vfinerr_grid.dat");
//	Interpolator2D valueInterp("src/grid/Vfin_only-top_grid.dat");
//	Interpolator2D errorInterp("src/grid/Vfinerr_only-top_grid.dat");


	std::cout << "sqrts pt vfin error" << std::endl;
	for (unsigned i = 0; i < sqrtsEval.size(); i++) {

		// Determine the minimum kinematically allowed sqrts for this pt value:
		const double minsqrts = sqrt(4.0*(mz*mz+ptEval[i]*ptEval[i]));

		if (sqrtsEval[i] >= minsqrts) {
			const double costh = sqrt(1.0 - (4.0*ptEval[i]*ptEval[i])/(-4.0*mz*mz + sqrtsEval[i]*sqrtsEval[i]));
			const double beta5 = pow(sqrt(1.0 - (4.0*mz*mz)/sqrtsEval[i]/sqrtsEval[i]),5);

			std::cout << sqrtsEval[i] << " " << ptEval[i] << " "
//			          << "(" << costh << " " << beta5 << ") "
			          << valueInterp.getZ(costh, beta5) << " "
			          << errorInterp.getZ(costh, beta5)
			          << std::endl;
		}
	}

	return 0;
}
